import os
import subprocess
import time
from pathlib import Path

import pytest
from pyinstrument.profiler import Profiler

from pyinferno import InfernoError, Renderer, flamegraph_from_lines, InfernoProfiler

TITLE = "My fantastic title"

SAMPLE_CODE = """
import time
def a():
    time.sleep(0.2)
a()"""


def test_simple_from_lines():
    testdata = Path("testdata/01_simple_from_lines")
    with open(testdata / "input.prof") as f:
        result = flamegraph_from_lines(f.readlines(), TITLE)
    if os.getenv("TEST_OVERWRITE"):
        with open(testdata / "output.svg", "w+") as f:
            f.write(result)
    with open(testdata / "output.svg") as f:
        contents = f.read().strip()
        assert result == contents
        assert TITLE in contents


def test_pyinstrument():
    p = Profiler()
    p.start()
    time.sleep(0.2)
    p.stop()
    result = p.output(Renderer(title=TITLE))
    assert len(result) > 0
    assert TITLE in result


def test_pyinstrument_default_title():
    p = Profiler()
    p.start()
    time.sleep(0.2)
    p.stop()
    result = p.output(Renderer())
    assert len(result) > 0


def test_pyinstrument_context_manager():
    with Profiler() as p:
        time.sleep(0.2)
    result = p.output(Renderer(title=TITLE))
    assert len(result) > 0
    assert TITLE in result


def test_pyinferno_context_manager(tmp_path):
    out_path = tmp_path / "out.svg"
    with InfernoProfiler(file=out_path, title=TITLE) as p:
        time.sleep(0.2)
    with open(out_path) as f:
        result = f.read()
    assert len(result) > 0
    assert TITLE in result


def test_pyinferno_context_manager_raises_with_no_args():
    with pytest.raises(ValueError):
        with InfernoProfiler() as p:
            time.sleep(0.2)


def test_pyinstrument_cli(tmp_path):
    code_path = tmp_path / "code.py"
    out_path = tmp_path / "out.svg"
    with open(code_path, "w+") as f:
        f.write(SAMPLE_CODE)
    subprocess.run([
        "pyinstrument",
        "-r", "pyinferno.Renderer",
        "-p", f"title={TITLE}",
        "-o",
        str(out_path.absolute()),
        str(code_path.absolute()),
    ], check=True)
    with open(out_path) as f:
        result = f.read()
    assert len(result) > 0
    assert TITLE in result


def test_pyinferno_cli(tmp_path):
    code_path = tmp_path / "code.py"
    out_path = tmp_path / "out.svg"
    with open(code_path, "w+") as f:
        f.write(SAMPLE_CODE)
    subprocess.run([
        "pyinferno",
        "-o",
        str(out_path.absolute()),
        str(code_path.absolute()),
    ], check=True)
    with open(out_path) as f:
        result = f.read()
    assert len(result) > 0
    assert "code.py" in result


def test_error_from_rust():
    with pytest.raises(InfernoError):
        flamegraph_from_lines(["not a valid trace"])



import argparse
import os
import subprocess
import sys
import tempfile
import webbrowser


def main():

    parser = argparse.ArgumentParser(usage="pyinferno [-o outfile] [pyinstrument_options] <script> [args]")
    parser.add_argument(
        "-o", "--outfile",
        action="store",
        dest="outfile",
        help="File to save the flamegraph to",
        default=None,
    )
    inferno_args, pyinstrument_args = parser.parse_known_args()
    if inferno_args.outfile:
        outpath = inferno_args.outfile
    else:
        outpath = tempfile.NamedTemporaryFile(prefix="flamegraph-", suffix=".svg", delete=False).name

    argv = sys.argv[1:]
    if not argv:
        parser.print_usage()
        sys.exit(0)

    args = [
        "pyinstrument",
        "-r", "pyinferno.Renderer",
        "-p", f"title={' '.join(pyinstrument_args)}",
        "-o", outpath,
        *pyinstrument_args
    ]
    subprocess.run(args, check=True)

    if not inferno_args.outfile:
        filepath = os.path.relpath(outpath)
        webbrowser.open(filepath)



from __future__ import annotations

from .pyinferno import InfernoError, flamegraph_from_lines

from pyinstrument.renderers.base import Renderer
from pyinstrument.session import Session


class InfernoRenderer(Renderer):
    output_file_extension = "svg"

    def __init__(self, title: str | None = None):
        self.title = title

    def should_skip(self, filename: str, function: str) -> bool:
        return filename.startswith("<frozen importlib._bootstrap")

    def render(self, session: Session) -> str:
        """
        Render a pyinstrument Session into an inferno flamegraph.
        """
        samples_per_s = session.sample_count/session.duration
        lines = []
        for record in session.frame_records:
            formatted_frames = []
            frames, time_spent = record
            for frame in frames:
                try:
                    identifier, _, attributes = frame.partition("\x01")
                    function, filename, lineno = identifier.split("\x00")
                except ValueError:
                    raise InfernoError(f"Could not parse frame '{frame}'")
                if not self.should_skip(filename, function):
                    formatted_frames.append(":".join((filename, lineno, function)))
            lines.append(f"{';'.join(formatted_frames)} {round(time_spent*samples_per_s)}")
        return flamegraph_from_lines(lines, self.title)



from __future__ import annotations

import contextlib
import os
import tempfile
import webbrowser
from pathlib import Path

from pyinstrument.profiler import Profiler

from .renderer import InfernoRenderer


@contextlib.contextmanager
def InfernoProfiler(
    file: str | Path | None = None,
    auto_open: bool = False,
    title: str | None = None,
    **kwargs,
):
    if file is None and not auto_open:
        raise ValueError(
            "Must pass a file to save the flamegraph to, or auto_open=True to open directly in the browser."
        )
    profiler = Profiler(**kwargs)
    profiler.start()
    try:
        yield profiler
    finally:
        profiler.stop()
        output = profiler.output(InfernoRenderer(title=title))

        if file:
            outpath = file
        else:
            outpath = tempfile.NamedTemporaryFile(prefix="flamegraph-", suffix=".svg", delete=False).name

        with open(outpath, "w+") as f:
            f.write(output)

        if auto_open:
            filepath = os.path.relpath(outpath)
            webbrowser.open(filepath)
